package de.uni.karlsruhe.aifb.prog2.chess;

/** Die abstrakte Implementierung der Schachfigur fuegt dem
  * urspruenglichen Interface eine Farbe (Schwarz oder Weiss)
  * hinzu und implementiert die canTake und toChar-Methoden.
  */
public abstract class AbstractChessman implements Chessman {


  
  /** Diese Zeichen wird zur Darstellung der Figur verwendet. 
    **/
  private char description;

  /** Dieser Boolesche Wert besagt, ob es sich um eine schwarze Figur
    * handelt.
    **/
  private boolean isBlack;

  /** Konstruktor.
    * @param isBlack ist true genau dann, wenn die Figur schwarz ist.
    * @param description eine Darstellung der Figur als Zeichen.
    *   Dieser Wert muss ein Grossbuchstabe (zwischen A und Z) sein.
    **/
  public AbstractChessman(boolean isBlack,char description) {
    this.description = description;
    this.isBlack = isBlack;
  }

  /** Liefert ein einzelnes Zeichen, das die Figur repraesentiert.
    **/
  public char toChar() {
    return (char) (description + ((isBlack) ? ('a' - 'A') : 0));
  }

  /** Testet, ob diese Figur eine andere Figur theoretisch zu schlagen
    * in der Lage ist. Dies ist immer der Fall, wenn die andere Figur
    * nicht dieselbe Farbe hat.
    **/
  public boolean canTake(Chessman chessman) {
    if (!(chessman instanceof AbstractChessman))
      return false;
    return isBlack != ((AbstractChessman)chessman).isBlack;
  }

  /** Gibt true genau dann zurueck,
    * wenn die Schachfigur schwarz ist
    **/
  public boolean isBlack() {
    return isBlack;
  }

  /** Hilfsmethode, die zur Programmierung der
    * getDestinations-methode verwendet werden kann.
    * @param destinations alle Felder, auf die die Figur ziehen
    *   kann, ohne dass der Weg dorthin von einer anderen Figur 
    *   blockiert wuerde (einschliesslich der blockierenden Figur
    *   selbst). Genau diese Eintraege im Booleschen Feld muessen dann
    *   auf true gesetzt sein. Die Methode setzt in diesem Feld exakt
    *   jene Werte auf false zurueck, die mit einer nicht schlagbaren 
    *   Figur besetzt sind.
    * @param chessboard das zugehoerige Schachbrett.
    * @param chessman die Schachfigur, die bewegt werden soll
    **/
  public static void cleanBlocked(boolean[][] destinations,
                   Chessboard chessboard,Chessman chessman) {
    for (int i = 0; i < destinations.length; i++)
      for (int j = 0; j < destinations[i].length; j++)
        if (destinations[i][j]) { // muss geprueft werden?
          Chessman opponent = chessboard.getContent(i,j);
          if (opponent != null && !chessman.canTake(opponent))
            destinations[i][j] = false;
        }
  }

  /** Vergleicht zwei Schachfiguren auf Typ-Gleichheit. Zwei
    * Schachfiguren gelten hierbei als gleich vom gleichen Typ,
    * wenn sie dieselbe Zeichendarstellung (toChar) haben.
    */
  public boolean equals(Object o) {
    if (o instanceof Chessman)
      return ((Chessman)o).toChar() == toChar();
    return false;
  }

  /** Da wir die equals-Methode ueberschrieben haben, muessen
    * wir auch die hashCode-Methode ueberschreiben.
    **/
  public int hashCode() {
    return toChar();
  }

}