package de.uni.karlsruhe.aifb.prog2.chess;

import Prog1Tools.GameEngine;

/** Einfache Schachapplikation. Stellt ein Schachbrett auf dem
  * Bildschirm dar, auf dem der Spieler Zuege machen kann.
  **/
public class Chessgame extends ChessboardDisplay {

  /** Die Reihe des letzten Buttons, der gedrueckt wurde. */
  private int lastRow;

  /** Die Spalte des letzten Buttons, der gedrueckt wurde. */
  private int lastCol;

  /** Ist momentan ein Button aktiviert? */
  private boolean activeButton;

  /** Die verwendeten Schachfiguren. */
  public final static Chessman[] CHESSMEN =
  {
    new Pawn(true),    //  0: schwarzer Bauer
    new Pawn(false),   //  1: weisser Bauer
    new Rook(true),    //  2: schwarzer Turm
    new Rook(false),   //  3: weisser Turm
    new Knight(true),  //  4: schwarzer Springer
    new Knight(false), //  5: weisser Springer
    new Bishop(true),  //  6: schwarzer Laeufer
    new Bishop(false), //  7: weisser Laeufer
    new Queen(true),   //  8: schwarze Dame
    new Queen(false),  //  9: weisse Dame
    new King(true),    // 10: schwarzer Koenig
    new King(false)    // 11: weisser Koenig
  };

  /** Der Standardaufbau eines Schachbretts */
  public final static int[][] BOARD_AT_START =
  {
    { 2, 4, 6, 8,10, 6, 4, 2},
    { 0, 0, 0, 0, 0, 0, 0, 0},
    {-1,-1,-1,-1,-1,-1,-1,-1},
    {-1,-1,-1,-1,-1,-1,-1,-1},
    {-1,-1,-1,-1,-1,-1,-1,-1},
    {-1,-1,-1,-1,-1,-1,-1,-1},
    { 1, 1, 1, 1, 1, 1, 1, 1},
    { 3, 5, 7, 9,11, 7, 5, 3}
  };

  /** Erzeugt ein neues, "frisches" Schachbrett */
  protected Chessboard createInitial()
  {
    Chessboard board = new Chessboard(8,8);
    for (int i = 0; i < 8; i++)
      for (int j = 0; j < 8; j++)
        if (BOARD_AT_START[i][j] > -1)
          board.setContent(i,j,CHESSMEN[BOARD_AT_START[i][j]]);
    return board;
  }

  /** Konstruktor.
    **/
  public Chessgame() {
    super(null);
    chessboard = createInitial();
  }

  /** Gibt den Text zurueck, der aktuell auf dem
    * Feuer-Button stehen soll.
    **/
  public String getFireLabel() {
    return "RESET BOARD";
  }

  /** Gibt den Text zurueck, der in der aktuellen
    * Runde im Meldefenster stehen soll. Defaultwert ist
    * ein leerer String.
    **/
  public String getMessages() {
    return "Game Of Chess";
  }

  /** Gibt den Namen des Spiels als String zurueck.
    **/
  public String getGameName() {
    return "Chessboard";
  }

  /** Signalisiert, dass ein bestimmter Button gedrueckt wurde.
    * Standardimplementierung ist es, nichts zu tun.
    * @param row die Zeile, von 0 an gezaehlt
    * @param col die Zeile, von 0 an gezaehlt
    **/
  public void buttonPressed(int row,int col) {
    // Falls erstmals auf eine Figur gedrueckt wird, merken
    // wir uns die Koordinaten
    if (!activeButton && chessboard.getContent(row,col) != null) {
      lastRow = row;
      lastCol = col;
      activeButton = true;
      return;
    }
    // Andernfalls teste, ob es sich um einen erlaubten Zug handelt
    if (activeButton)
    {
      Chessman chessman = chessboard.getContent(lastRow,lastCol);
      boolean[][] destinations =
        chessman.getDestinations(chessboard,lastRow,lastCol);
      if (destinations[row][col]) { // bewege Figur
        chessboard.move(lastRow,lastCol,row,col);
      }
    }
    // Zu guter Letzt, loesche die Selektion
    activeButton = false;
  }

  /** Signalisert, dass; der Feuer-Button gedrueckt wurde.
    * Versetzt das Brett wieder in den Anfangszustand.
    **/
  public void firePressed() {
    chessboard = createInitial();
    activeButton = false;
  }

  /** Main-Methode zum Starten des Programms */
  public static void main(String[] args) {
    new GameEngine(new Chessgame());
  }

}