package de.uni.karlsruhe.aifb.prog2.chess;

/** Die Bauern-Figur
  */
public class Pawn extends AbstractChessman {

  /** Konstruktor.
    * @param isBlack ist true genau dann, wenn die Figur schwarz ist.
    **/
  public Pawn(boolean isBlack) {
    super(isBlack,'P');
  }

  /** Ueberprueft, auf welche Felder sich die Figur im naechsten Zug
    * theoretisch bewegen kann.
    **/
  public boolean[][] getDestinations
    (Chessboard board,int row,int col) {
    // Bis das Gegenteil bewiesen ist, darf unsere Figur nirgends hin
    boolean[][] res =
      new boolean[board.getNumOfRows()]
                 [board.getNumOfColumns()];
    // Im Falle "schwarz" gehen wir nach unten, andernfalls nach oben
    row = isBlack() ? row + 1 : row - 1;
    // Pruefe, ob wir noch im Feld sind
    if (!board.isValid(row,col))
      return res;
    // Pruefe nun, ob der Bauer geradeaus gehen kann
    if (board.getContent(row,col) == null)
      res[row][col] = true;
    // Pruefe nun, ob der Bauer schlagen kann
    for (int i = -1; i < 2; i += 2) {
      int column = col + i;
      if (!board.isValid(row,column))
        continue;
      Chessman opponent
        = board.getContent(row,column);
      if (opponent != null && canTake(opponent))
        res[row][column] = true;
    }
    // Pruefe, ob der Bauer zwei Felder gehen darf
    // Voraussetzung: er darf zumindest EIN Feld
    // geradeaus gehen.
    if (res[row][col]) {
      row = isBlack() ? row + 1 : row - 1;
      if (board.isValid(row,col) &&
          ((row == res.length - 4 && !isBlack())
          || (row == 3 && isBlack())) &&
          board.getContent(row,col) == null)
        res[row][col] = true;
    }
    // Fertig!
    return res;
  }


}